@@ -29,7 +29,6 @@ from sentry_sdk.utils import ContextVar
 from sentry_sdk.sessions import SessionFlusher
 from sentry_sdk.envelope import Envelope
 from sentry_sdk.profiler import setup_profiler
-from sentry_sdk.tracing_utils import has_tracestate_enabled, reinflate_tracestate
 
 from sentry_sdk._types import MYPY
 
@@ -90,6 +89,17 @@ def _get_options(*args, **kwargs):
     if rv["instrumenter"] is None:
         rv["instrumenter"] = INSTRUMENTER.SENTRY
 
+    if rv["project_root"] is None:
+        try:
+            project_root = os.getcwd()
+        except Exception:
+            project_root = None
+
+        rv["project_root"] = project_root
+
+    if rv["enable_tracing"] is True and rv["traces_sample_rate"] is None:
+        rv["traces_sample_rate"] = 1.0
+
     return rv
 
 
@@ -103,6 +113,7 @@ class _Client(object):
     def __init__(self, *args, **kwargs):
         # type: (*Any, **Any) -> None
         self.options = get_options(*args, **kwargs)  # type: Dict[str, Any]
+
         self._init_impl()
 
     def __getstate__(self):
@@ -222,7 +233,10 @@ class _Client(object):
             event["platform"] = "python"
 
         event = handle_in_app(
-            event, self.options["in_app_exclude"], self.options["in_app_include"]
+            event,
+            self.options["in_app_exclude"],
+            self.options["in_app_include"],
+            self.options["project_root"],
         )
 
         # Postprocess the event here so that annotated types do
@@ -410,13 +424,6 @@ class _Client(object):
 
         attachments = hint.get("attachments")
 
-        # this is outside of the `if` immediately below because even if we don't
-        # use the value, we want to make sure we remove it before the event is
-        # sent
-        raw_tracestate = (
-            event_opt.get("contexts", {}).get("trace", {}).pop("tracestate", "")
-        )
-
         dynamic_sampling_context = (
             event_opt.get("contexts", {})
             .get("trace", {})
@@ -432,14 +439,7 @@ class _Client(object):
                 "sent_at": format_timestamp(datetime.utcnow()),
             }
 
-            if has_tracestate_enabled():
-                tracestate_data = raw_tracestate and reinflate_tracestate(
-                    raw_tracestate.replace("sentry=", "")
-                )
-
-                if tracestate_data:
-                    headers["trace"] = tracestate_data
-            elif dynamic_sampling_context:
+            if dynamic_sampling_context:
                 headers["trace"] = dynamic_sampling_context
 
             envelope = Envelope(headers=headers)
