@@ -1,6 +1,5 @@
 import re
 import contextlib
-import json
 import math
 
 from numbers import Real
@@ -13,10 +12,7 @@ from sentry_sdk.utils import (
     capture_internal_exceptions,
     Dsn,
     logger,
-    safe_str,
-    to_base64,
     to_string,
-    from_base64,
 )
 from sentry_sdk._compat import PY2, iteritems
 from sentry_sdk._types import MYPY
@@ -31,10 +27,10 @@ else:
 if MYPY:
     import typing
 
-    from typing import Generator
-    from typing import Optional
     from typing import Any
     from typing import Dict
+    from typing import Generator
+    from typing import Optional
     from typing import Union
 
 
@@ -57,27 +53,6 @@ base64_stripped = (
     "([a-zA-Z0-9+/]{2,3})?"
 )
 
-# comma-delimited list of entries of the form `xxx=yyy`
-tracestate_entry = "[^=]+=[^=]+"
-TRACESTATE_ENTRIES_REGEX = re.compile(
-    # one or more xxxxx=yyyy entries
-    "^({te})+"
-    # each entry except the last must be followed by a comma
-    "(,|$)".format(te=tracestate_entry)
-)
-
-# this doesn't check that the value is valid, just that there's something there
-# of the form `sentry=xxxx`
-SENTRY_TRACESTATE_ENTRY_REGEX = re.compile(
-    # either sentry is the first entry or there's stuff immediately before it,
-    # ending in a comma (this prevents matching something like `coolsentry=xxx`)
-    "(?:^|.+,)"
-    # sentry's part, not including the potential comma
-    "(sentry=[^,]*)"
-    # either there's a comma and another vendor's entry or we end
-    "(?:,.+|$)"
-)
-
 
 class EnvironHeaders(Mapping):  # type: ignore
     def __init__(
@@ -114,12 +89,14 @@ def has_tracing_enabled(options):
     # type: (Dict[str, Any]) -> bool
     """
     Returns True if either traces_sample_rate or traces_sampler is
-    defined, False otherwise.
+    defined and enable_tracing is set and not false.
     """
-
     return bool(
-        options.get("traces_sample_rate") is not None
-        or options.get("traces_sampler") is not None
+        options.get("enable_tracing") is not False
+        and (
+            options.get("traces_sample_rate") is not None
+            or options.get("traces_sampler") is not None
+        )
     )
 
 
@@ -246,143 +223,6 @@ def extract_sentrytrace_data(header):
     }
 
 
-def extract_tracestate_data(header):
-    # type: (Optional[str]) -> typing.Mapping[str, Optional[str]]
-    """
-    Extracts the sentry tracestate value and any third-party data from the given
-    tracestate header, returning a dictionary of data.
-    """
-    sentry_entry = third_party_entry = None
-    before = after = ""
-
-    if header:
-        # find sentry's entry, if any
-        sentry_match = SENTRY_TRACESTATE_ENTRY_REGEX.search(header)
-
-        if sentry_match:
-            sentry_entry = sentry_match.group(1)
-
-            # remove the commas after the split so we don't end up with
-            # `xxx=yyy,,zzz=qqq` (double commas) when we put them back together
-            before, after = map(lambda s: s.strip(","), header.split(sentry_entry))
-
-            # extract sentry's value from its entry and test to make sure it's
-            # valid; if it isn't, discard the entire entry so that a new one
-            # will be created
-            sentry_value = sentry_entry.replace("sentry=", "")
-            if not re.search("^{b64}$".format(b64=base64_stripped), sentry_value):
-                sentry_entry = None
-        else:
-            after = header
-
-        # if either part is invalid or empty, remove it before gluing them together
-        third_party_entry = (
-            ",".join(filter(TRACESTATE_ENTRIES_REGEX.search, [before, after])) or None
-        )
-
-    return {
-        "sentry_tracestate": sentry_entry,
-        "third_party_tracestate": third_party_entry,
-    }
-
-
-def compute_tracestate_value(data):
-    # type: (typing.Mapping[str, str]) -> str
-    """
-    Computes a new tracestate value using the given data.
-
-    Note: Returns just the base64-encoded data, NOT the full `sentry=...`
-    tracestate entry.
-    """
-
-    tracestate_json = json.dumps(data, default=safe_str)
-
-    # Base64-encoded strings always come out with a length which is a multiple
-    # of 4. In order to achieve this, the end is padded with one or more `=`
-    # signs. Because the tracestate standard calls for using `=` signs between
-    # vendor name and value (`sentry=xxx,dogsaregreat=yyy`), to avoid confusion
-    # we strip the `=`
-    return (to_base64(tracestate_json) or "").rstrip("=")
-
-
-def compute_tracestate_entry(span):
-    # type: (Span) -> Optional[str]
-    """
-    Computes a new sentry tracestate for the span. Includes the `sentry=`.
-
-    Will return `None` if there's no client and/or no DSN.
-    """
-    data = {}
-
-    hub = span.hub or sentry_sdk.Hub.current
-
-    client = hub.client
-    scope = hub.scope
-
-    if client and client.options.get("dsn"):
-        options = client.options
-        user = scope._user
-
-        data = {
-            "trace_id": span.trace_id,
-            "environment": options["environment"],
-            "release": options.get("release"),
-            "public_key": Dsn(options["dsn"]).public_key,
-        }
-
-        if user and (user.get("id") or user.get("segment")):
-            user_data = {}
-
-            if user.get("id"):
-                user_data["id"] = user["id"]
-
-            if user.get("segment"):
-                user_data["segment"] = user["segment"]
-
-            data["user"] = user_data
-
-        if span.containing_transaction:
-            data["transaction"] = span.containing_transaction.name
-
-        return "sentry=" + compute_tracestate_value(data)
-
-    return None
-
-
-def reinflate_tracestate(encoded_tracestate):
-    # type: (str) -> typing.Optional[Mapping[str, str]]
-    """
-    Given a sentry tracestate value in its encoded form, translate it back into
-    a dictionary of data.
-    """
-    inflated_tracestate = None
-
-    if encoded_tracestate:
-        # Base64-encoded strings always come out with a length which is a
-        # multiple of 4. In order to achieve this, the end is padded with one or
-        # more `=` signs. Because the tracestate standard calls for using `=`
-        # signs between vendor name and value (`sentry=xxx,dogsaregreat=yyy`),
-        # to avoid confusion we strip the `=` when the data is initially
-        # encoded. Python's decoding function requires they be put back.
-        # Fortunately, it doesn't complain if there are too many, so we just
-        # attach two `=` on spec (there will never be more than 2, see
-        # https://en.wikipedia.org/wiki/Base64#Decoding_Base64_without_padding).
-        tracestate_json = from_base64(encoded_tracestate + "==")
-
-        try:
-            assert tracestate_json is not None
-            inflated_tracestate = json.loads(tracestate_json)
-        except Exception as err:
-            logger.warning(
-                (
-                    "Unable to attach tracestate data to envelope header: {err}"
-                    + "\nTracestate value is {encoded_tracestate}"
-                ).format(err=err, encoded_tracestate=encoded_tracestate),
-            )
-
-    return inflated_tracestate
-
-
 def _format_sql(cursor, sql):
     # type: (Any, str) -> Optional[str]
 
@@ -403,22 +243,6 @@ def _format_sql(cursor, sql):
     return real_sql or to_string(sql)
 
 
-def has_tracestate_enabled(span=None):
-    # type: (Optional[Span]) -> bool
-
-    client = ((span and span.hub) or sentry_sdk.Hub.current).client
-    options = client and client.options
-
-    return bool(options and options["_experiments"].get("propagate_tracestate"))
-
-
-def has_custom_measurements_enabled():
-    # type: () -> bool
-    client = sentry_sdk.Hub.current.client
-    options = client and client.options
-    return bool(options and options["_experiments"].get("custom_measurements"))
-
-
 class Baggage(object):
     __slots__ = ("sentry_items", "third_party_items", "mutable")
 
@@ -552,6 +376,25 @@ class Baggage(object):
         return ",".join(items)
 
 
+def should_propagate_trace(hub, url):
+    # type: (sentry_sdk.Hub, str) -> bool
+    """
+    Returns True if url matches trace_propagation_targets configured in the given hub. Otherwise, returns False.
+    """
+    client = hub.client  # type: Any
+    trace_propagation_targets = client.options["trace_propagation_targets"]
+
+    if trace_propagation_targets is None:
+        return False
+
+    for target in trace_propagation_targets:
+        matched = re.search(target, url)
+        if matched:
+            return True
+
+    return False
+
+
 # Circular imports
 from sentry_sdk.tracing import LOW_QUALITY_TRANSACTION_SOURCES
 
